#!/usr/bin/env python

import codecs
import io
import unittest

import idna.codec

CODEC_NAME = "idna2008"


class IDNACodecTests(unittest.TestCase):
    def setUp(self):
        from . import test_idna

        self.idnatests = test_idna.IDNATests()
        self.idnatests.setUp()

    def testCodec(self):
        self.assertIs(codecs.lookup(CODEC_NAME).incrementalencoder, idna.codec.IncrementalEncoder)

    def testDirectDecode(self):
        self.idnatests.test_decode(decode=lambda obj: codecs.decode(obj, CODEC_NAME))

    def testIndirectDecode(self):
        self.idnatests.test_decode(decode=lambda obj: obj.decode(CODEC_NAME), skip_str=True)

    def testDirectEncode(self):
        self.idnatests.test_encode(encode=lambda obj: codecs.encode(obj, CODEC_NAME))

    def testIndirectEncode(self):
        self.idnatests.test_encode(encode=lambda obj: obj.encode(CODEC_NAME), skip_bytes=True)

    def testStreamReader(self):
        def decode(obj):
            if isinstance(obj, str):
                obj = bytes(obj, "ascii")
            buffer = io.BytesIO(obj)
            stream = codecs.getreader(CODEC_NAME)(buffer)
            return stream.read()

        return self.idnatests.test_decode(decode=decode, skip_str=True)

    def testStreamWriter(self):
        def encode(obj):
            buffer = io.BytesIO()
            stream = codecs.getwriter(CODEC_NAME)(buffer)
            stream.write(obj)
            stream.flush()
            return buffer.getvalue()

        return self.idnatests.test_encode(encode=encode)

    def testIncrementalDecoder(self):
        # Tests derived from Python standard library test/test_codecs.py

        incremental_tests = (
            ("python.org", b"python.org"),
            ("python.org.", b"python.org."),
            ("pyth\xf6n.org", b"xn--pythn-mua.org"),
            ("pyth\xf6n.org.", b"xn--pythn-mua.org."),
        )

        for decoded, encoded in incremental_tests:
            self.assertEqual(
                "".join(codecs.iterdecode((bytes([c]) for c in encoded), CODEC_NAME)),
                decoded,
            )

        decoder = codecs.getincrementaldecoder(CODEC_NAME)()
        self.assertEqual(
            decoder.decode(
                b"xn--xam",
            ),
            "",
        )
        self.assertEqual(
            decoder.decode(
                b"ple-9ta.o",
            ),
            "\xe4xample.",
        )
        self.assertEqual(decoder.decode(b"rg"), "")
        self.assertEqual(decoder.decode(b"", True), "org")

        decoder.reset()
        self.assertEqual(
            decoder.decode(
                b"xn--xam",
            ),
            "",
        )
        self.assertEqual(
            decoder.decode(
                b"ple-9ta.o",
            ),
            "\xe4xample.",
        )
        self.assertEqual(decoder.decode(b"rg."), "org.")
        self.assertEqual(decoder.decode(b"", True), "")

    def testIncrementalEncoder(self):
        # Tests derived from Python standard library test/test_codecs.py

        incremental_tests = (
            ("python.org", b"python.org"),
            ("python.org.", b"python.org."),
            ("pyth\xf6n.org", b"xn--pythn-mua.org"),
            ("pyth\xf6n.org.", b"xn--pythn-mua.org."),
        )
        for decoded, encoded in incremental_tests:
            self.assertEqual(b"".join(codecs.iterencode(decoded, CODEC_NAME)), encoded)

        encoder = codecs.getincrementalencoder(CODEC_NAME)()
        self.assertEqual(encoder.encode("\xe4x"), b"")
        self.assertEqual(encoder.encode("ample.org"), b"xn--xample-9ta.")
        self.assertEqual(encoder.encode("", True), b"org")

        encoder.reset()
        self.assertEqual(encoder.encode("\xe4x"), b"")
        self.assertEqual(encoder.encode("ample.org."), b"xn--xample-9ta.org.")
        self.assertEqual(encoder.encode("", True), b"")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import unittest
import warnings

import idna


class IDNATests(unittest.TestCase):
    def setUp(self):
        self.tld_strings: list[tuple[str, bytes]] = [
            ("\u6d4b\u8bd5", b"xn--0zwm56d"),
            ("\u092a\u0930\u0940\u0915\u094d\u0937\u093e", b"xn--11b5bs3a9aj6g"),
            ("\ud55c\uad6d", b"xn--3e0b707e"),
            ("\u09ad\u09be\u09b0\u09a4", b"xn--45brj9c"),
            ("\u09ac\u09be\u0982\u09b2\u09be", b"xn--54b7fta0cc"),
            (
                "\u0438\u0441\u043f\u044b\u0442\u0430\u043d\u0438\u0435",
                b"xn--80akhbyknj4f",
            ),
            ("\u0441\u0440\u0431", b"xn--90a3ac"),
            ("\ud14c\uc2a4\ud2b8", b"xn--9t4b11yi5a"),
            (
                "\u0b9a\u0bbf\u0b99\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0bc2\u0bb0\u0bcd",
                b"xn--clchc0ea0b2g2a9gcd",
            ),
            ("\u05d8\u05e2\u05e1\u05d8", b"xn--deba0ad"),
            ("\u4e2d\u56fd", b"xn--fiqs8s"),
            ("\u4e2d\u570b", b"xn--fiqz9s"),
            ("\u0c2d\u0c3e\u0c30\u0c24\u0c4d", b"xn--fpcrj9c3d"),
            ("\u0dbd\u0d82\u0d9a\u0dcf", b"xn--fzc2c9e2c"),
            ("\u6e2c\u8a66", b"xn--g6w251d"),
            ("\u0aad\u0abe\u0ab0\u0aa4", b"xn--gecrj9c"),
            ("\u092d\u093e\u0930\u0924", b"xn--h2brj9c"),
            ("\u0622\u0632\u0645\u0627\u06cc\u0634\u06cc", b"xn--hgbk6aj7f53bba"),
            ("\u0baa\u0bb0\u0bbf\u0b9f\u0bcd\u0b9a\u0bc8", b"xn--hlcj6aya9esc7a"),
            ("\u0443\u043a\u0440", b"xn--j1amh"),
            ("\u9999\u6e2f", b"xn--j6w193g"),
            ("\u03b4\u03bf\u03ba\u03b9\u03bc\u03ae", b"xn--jxalpdlp"),
            ("\u0625\u062e\u062a\u0628\u0627\u0631", b"xn--kgbechtv"),
            ("\u53f0\u6e7e", b"xn--kprw13d"),
            ("\u53f0\u7063", b"xn--kpry57d"),
            ("\u0627\u0644\u062c\u0632\u0627\u0626\u0631", b"xn--lgbbat1ad8j"),
            ("\u0639\u0645\u0627\u0646", b"xn--mgb9awbf"),
            ("\u0627\u06cc\u0631\u0627\u0646", b"xn--mgba3a4f16a"),
            ("\u0627\u0645\u0627\u0631\u0627\u062a", b"xn--mgbaam7a8h"),
            ("\u067e\u0627\u06a9\u0633\u062a\u0627\u0646", b"xn--mgbai9azgqp6j"),
            ("\u0627\u0644\u0627\u0631\u062f\u0646", b"xn--mgbayh7gpa"),
            ("\u0628\u06be\u0627\u0631\u062a", b"xn--mgbbh1a71e"),
            ("\u0627\u0644\u0645\u063a\u0631\u0628", b"xn--mgbc0a9azcg"),
            ("\u0627\u0644\u0633\u0639\u0648\u062f\u064a\u0629", b"xn--mgberp4a5d4ar"),
            ("\u10d2\u10d4", b"xn--node"),
            ("\u0e44\u0e17\u0e22", b"xn--o3cw4h"),
            ("\u0633\u0648\u0631\u064a\u0629", b"xn--ogbpf8fl"),
            ("\u0440\u0444", b"xn--p1ai"),
            ("\u062a\u0648\u0646\u0633", b"xn--pgbs0dh"),
            ("\u0a2d\u0a3e\u0a30\u0a24", b"xn--s9brj9c"),
            ("\u0645\u0635\u0631", b"xn--wgbh1c"),
            ("\u0642\u0637\u0631", b"xn--wgbl6a"),
            ("\u0b87\u0bb2\u0b99\u0bcd\u0b95\u0bc8", b"xn--xkc2al3hye2a"),
            ("\u0b87\u0ba8\u0bcd\u0ba4\u0bbf\u0baf\u0bbe", b"xn--xkc2dl3a5ee0h"),
            ("\u65b0\u52a0\u5761", b"xn--yfro4i67o"),
            ("\u0641\u0644\u0633\u0637\u064a\u0646", b"xn--ygbi2ammx"),
            ("\u30c6\u30b9\u30c8", b"xn--zckzah"),
            ("\u049b\u0430\u0437", b"xn--80ao21a"),
            ("\u0645\u0644\u064a\u0633\u064a\u0627", b"xn--mgbx4cd0ab"),
            ("\u043c\u043e\u043d", b"xn--l1acc"),
            ("\u0633\u0648\u062f\u0627\u0646", b"xn--mgbpl2fh"),
        ]

    def testIDNTLDALabels(self):
        for u, a in self.tld_strings:
            self.assertEqual(a, idna.alabel(u))

    def testIDNTLDULabels(self):
        for u, a in self.tld_strings:
            self.assertEqual(u, idna.ulabel(a))

    def test_valid_label_length(self):
        self.assertTrue(idna.valid_label_length("a" * 63))
        self.assertFalse(idna.valid_label_length("a" * 64))
        self.assertRaises(idna.IDNAError, idna.encode, "a" * 64)

    def test_oversized_input_rejected_promptly(self):
        # GHSA-65pc-fj4g-8rjx: encode/decode must reject inputs that
        # exceed the maximum DNS domain length before per-codepoint
        # validation runs, so labels dominated by CONTEXTO codepoints
        # cannot drive validation into quadratic time.
        import time

        for payload in ("٠" * 8000, "・" * 8000 + "漢"):
            start = time.perf_counter()
            self.assertRaises(idna.IDNAError, idna.encode, payload)
            self.assertRaises(idna.IDNAError, idna.decode, payload)
            self.assertLess(time.perf_counter() - start, 1.0)

    def test_oversized_label_rejected_promptly(self):
        # The whole-domain cap in encode()/decode() does not cover direct
        # callers of alabel/ulabel/check_label, nor the idna2008
        # incremental codec which calls alabel/ulabel per label. Without a
        # per-label cap, a single oversized CONTEXTO-heavy label still
        # drives validation into quadratic time.
        import codecs
        import time

        import idna.codec  # register the idna2008 codec

        payload = "・" * 8000 + "漢"
        start = time.perf_counter()
        self.assertRaises(idna.IDNAError, idna.check_label, payload)
        self.assertRaises(idna.IDNAError, idna.alabel, payload)
        self.assertRaises(idna.IDNAError, idna.ulabel, payload)
        self.assertRaises(
            idna.IDNAError,
            codecs.getincrementalencoder("idna2008")().encode,
            payload,
            True,
        )
        self.assertLess(time.perf_counter() - start, 1.0)

    def test_check_bidi(self):
        la = "\u0061"
        r = "\u05d0"
        al = "\u0627"
        an = "\u0660"
        en = "\u0030"
        es = "\u002d"
        cs = "\u002c"
        et = "\u0024"
        on = "\u0021"
        bn = "\u200c"
        nsm = "\u0610"
        ws = "\u0020"

        # RFC 5893 Rule 1
        self.assertTrue(idna.check_bidi(la))
        self.assertTrue(idna.check_bidi(r))
        self.assertTrue(idna.check_bidi(al))
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, an)

        # RFC 5893 Rule 2
        self.assertTrue(idna.check_bidi(r + al))
        self.assertTrue(idna.check_bidi(r + al))
        self.assertTrue(idna.check_bidi(r + an))
        self.assertTrue(idna.check_bidi(r + en))
        self.assertTrue(idna.check_bidi(r + es + al))
        self.assertTrue(idna.check_bidi(r + cs + al))
        self.assertTrue(idna.check_bidi(r + et + al))
        self.assertTrue(idna.check_bidi(r + on + al))
        self.assertTrue(idna.check_bidi(r + bn + al))
        self.assertTrue(idna.check_bidi(r + nsm))
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, r + la)
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, r + ws)

        # RFC 5893 Rule 3
        self.assertTrue(idna.check_bidi(r + al))
        self.assertTrue(idna.check_bidi(r + en))
        self.assertTrue(idna.check_bidi(r + an))
        self.assertTrue(idna.check_bidi(r + nsm))
        self.assertTrue(idna.check_bidi(r + nsm + nsm))
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, r + on)

        # RFC 5893 Rule 4
        self.assertTrue(idna.check_bidi(r + en))
        self.assertTrue(idna.check_bidi(r + an))
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, r + en + an)
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, r + an + en)

        # RFC 5893 Rule 5
        self.assertTrue(idna.check_bidi(la + en, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + es + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + cs + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + et + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + on + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + bn + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + nsm, check_ltr=True))

        # RFC 5893 Rule 6
        self.assertTrue(idna.check_bidi(la + la, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + en, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + en + nsm, check_ltr=True))
        self.assertTrue(idna.check_bidi(la + en + nsm + nsm, check_ltr=True))
        self.assertRaises(idna.IDNABidiError, idna.check_bidi, la + cs, check_ltr=True)

    def test_check_initial_combiner(self):
        m = "\u0300"
        a = "\u0061"

        self.assertTrue(idna.check_initial_combiner(a))
        self.assertTrue(idna.check_initial_combiner(a + m))
        self.assertRaises(idna.IDNAError, idna.check_initial_combiner, m + a)

    def test_check_hyphen_ok(self):
        self.assertTrue(idna.check_hyphen_ok("abc"))
        self.assertTrue(idna.check_hyphen_ok("a--b"))
        self.assertRaises(idna.IDNAError, idna.check_hyphen_ok, "aa--")
        self.assertRaises(idna.IDNAError, idna.check_hyphen_ok, "a-")
        self.assertRaises(idna.IDNAError, idna.check_hyphen_ok, "-a")

    def test_valid_contextj(self):
        zwnj = "\u200c"
        zwj = "\u200d"
        virama = "\u094d"
        latin = "\u0061"

        # RFC 5892 Appendix A.1 (Zero Width Non-Joiner)
        self.assertFalse(idna.valid_contextj(zwnj, 0))
        self.assertFalse(idna.valid_contextj(latin + zwnj, 1))  # No preceding Virama
        self.assertTrue(idna.valid_contextj(virama + zwnj, 1))  # Preceding Virama

        # RFC 5892 Appendix A.2 (Zero Width Joiner)
        self.assertFalse(idna.valid_contextj(zwj, 0))
        self.assertFalse(idna.valid_contextj(latin + zwj, 1))  # No preceding Virama
        self.assertTrue(idna.valid_contextj(virama + zwj, 1))  # Preceding Virama

    def test_valid_contexto(self):
        latin = "\u0061"
        latin_l = "\u006c"
        greek = "\u03b1"
        hebrew = "\u05d0"
        katakana = "\u30a1"
        hiragana = "\u3041"
        han = "\u6f22"
        arabic_digit = "\u0660"
        ext_arabic_digit = "\u06f0"

        # RFC 5892 Rule A.3 (Middle Dot)
        latin_middle_dot = "\u00b7"
        self.assertTrue(idna.valid_contexto(latin_l + latin_middle_dot + latin_l, 1))
        self.assertFalse(idna.valid_contexto(latin_middle_dot + latin_l, 1))
        self.assertFalse(idna.valid_contexto(latin_l + latin_middle_dot, 0))
        self.assertFalse(idna.valid_contexto(latin_middle_dot, 0))
        self.assertFalse(idna.valid_contexto(latin_l + latin_middle_dot + latin, 1))

        # RFC 5892 Rule A.4 (Greek Lower Numeral Sign)
        glns = "\u0375"
        self.assertTrue(idna.valid_contexto(glns + greek, 0))
        self.assertFalse(idna.valid_contexto(glns + latin, 0))
        self.assertFalse(idna.valid_contexto(glns, 0))
        self.assertFalse(idna.valid_contexto(greek + glns, 1))

        # RFC 5892 Rule A.5 (Hebrew Punctuation Geresh)
        geresh = "\u05f3"
        self.assertTrue(idna.valid_contexto(hebrew + geresh, 1))
        self.assertFalse(idna.valid_contexto(latin + geresh, 1))

        # RFC 5892 Rule A.6 (Hebrew Punctuation Gershayim)
        gershayim = "\u05f4"
        self.assertTrue(idna.valid_contexto(hebrew + gershayim, 1))
        self.assertFalse(idna.valid_contexto(latin + gershayim, 1))

        # RFC 5892 Rule A.7 (Katakana Middle Dot)
        ja_middle_dot = "\u30fb"
        self.assertTrue(idna.valid_contexto(katakana + ja_middle_dot + katakana, 1))
        self.assertTrue(idna.valid_contexto(hiragana + ja_middle_dot + hiragana, 1))
        self.assertTrue(idna.valid_contexto(han + ja_middle_dot + han, 1))
        self.assertTrue(idna.valid_contexto(han + ja_middle_dot + latin, 1))
        self.assertTrue(idna.valid_contexto("\u6f22\u30fb\u5b57", 1))
        self.assertFalse(idna.valid_contexto("\u0061\u30fb\u0061", 1))

        # RFC 5892 Rule A.8 (Arabic-Indic Digits)
        self.assertTrue(idna.valid_contexto(arabic_digit + arabic_digit, 0))
        self.assertFalse(idna.valid_contexto(arabic_digit + ext_arabic_digit, 0))

        # RFC 5892 Rule A.9 (Extended Arabic-Indic Digits)
        self.assertTrue(idna.valid_contexto(ext_arabic_digit + ext_arabic_digit, 0))
        self.assertFalse(idna.valid_contexto(ext_arabic_digit + arabic_digit, 0))

    def test_encode(self, encode=None, skip_bytes=False):
        if encode is None:
            encode = idna.encode

        self.assertEqual(encode("xn--zckzah.xn--zckzah"), b"xn--zckzah.xn--zckzah")
        self.assertEqual(encode("\u30c6\u30b9\u30c8.xn--zckzah"), b"xn--zckzah.xn--zckzah")
        self.assertEqual(encode("\u30c6\u30b9\u30c8.\u30c6\u30b9\u30c8"), b"xn--zckzah.xn--zckzah")
        self.assertEqual(encode("abc.abc"), b"abc.abc")
        self.assertEqual(encode("xn--zckzah.abc"), b"xn--zckzah.abc")
        self.assertEqual(encode("\u30c6\u30b9\u30c8.abc"), b"xn--zckzah.abc")
        self.assertEqual(
            encode("\u0521\u0525\u0523-\u0523\u0523-----\u0521\u0523\u0523\u0523.aa"),
            b"xn---------90gglbagaar.aa",
        )
        if encode is idna.encode:
            self.assertRaises(
                idna.IDNAError,
                encode,
                "\u0521\u0524\u0523-\u0523\u0523-----\u0521\u0523\u0523\u0523.aa",
                uts46=False,
            )
        self.assertEqual(encode("a" * 63), b"a" * 63)
        self.assertRaises(idna.IDNAError, encode, "a" * 64)
        self.assertRaises(idna.core.InvalidCodepoint, encode, "*")
        if not skip_bytes:
            self.assertRaises(idna.IDNAError, encode, b"\x0a\x33\x81")

    def test_decode(self, decode=None, skip_str=False):
        if decode is None:
            decode = idna.decode
        self.assertEqual(decode(b"xn--zckzah.xn--zckzah"), "\u30c6\u30b9\u30c8.\u30c6\u30b9\u30c8")
        self.assertEqual(
            decode(b"xn--d1acufc.xn--80akhbyknj4f"),
            "\u0434\u043e\u043c\u0435\u043d.\u0438\u0441\u043f\u044b\u0442\u0430\u043d\u0438\u0435",
        )
        if not skip_str:
            self.assertEqual(
                decode("\u30c6\u30b9\u30c8.xn--zckzah"),
                "\u30c6\u30b9\u30c8.\u30c6\u30b9\u30c8",
            )
            self.assertEqual(
                decode("\u30c6\u30b9\u30c8.\u30c6\u30b9\u30c8"),
                "\u30c6\u30b9\u30c8.\u30c6\u30b9\u30c8",
            )
            self.assertEqual(decode("abc.abc"), "abc.abc")
        self.assertEqual(
            decode(b"xn---------90gglbagaar.aa"),
            "\u0521\u0525\u0523-\u0523\u0523-----\u0521\u0523\u0523\u0523.aa",
        )
        self.assertRaises(idna.IDNAError, decode, b"XN---------90GGLBAGAAC.AA")
        self.assertRaises(idna.IDNAError, decode, b"xn---------90gglbagaac.aa")
        self.assertRaises(idna.IDNAError, decode, b"xn--")
        self.assertRaises(idna.IDNAError, decode, b"\x8d\xd2")
        self.assertRaises(
            idna.IDNAError,
            decode,
            b"A.A.0.a.a.A.0.a.A.A.0.a.A.0A.2.a.A.A.0.a.A.0.A.a.A0.a.a.A.0.a.fB.A.A.a.A.A.B.A.A.a.A.A.B.A.A.a.A.A.0.a.A.a.a.A.A.0.a.A.0.A.a.A0.a.a.A.0.a.fB.A.A.a.A.A.B.0A.A.a.A.A.B.A.A.a.A.A.a.A.A.B.A.A.a.A.0.a.B.A.A.a.A.B.A.a.A.A.5.a.A.0.a.Ba.A.B.A.A.a.A.0.a.Xn--B.A.A.A.a",
        )
        self.assertRaises(idna.IDNAError, decode, b"xn--ukba655qaaaa14431eeaaba.c")

    def test_encode_transitional_deprecation_warning(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            idna.encode("example.com", uts46=True, transitional=True)
            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))
            self.assertIn("transitional", str(w[0].message).lower())

    def test_encode_no_transitional_no_warning(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            idna.encode("example.com", uts46=True)
            self.assertEqual(len(w), 0)

    def test_encode_decode_invalid_input_type(self):
        # encode() and decode() are documented to raise IDNAError on bad
        # input.  Inputs that are not str, bytes, or bytes-like used to leak
        # a raw TypeError out of str(s, "ascii"); they should be wrapped in
        # IDNAError just like UnicodeDecodeError already is.
        for value in (42, None, 1.5, ["a", "b"], {"a": 1}):
            self.assertRaises(idna.IDNAError, idna.encode, value)
            self.assertRaises(idna.IDNAError, idna.decode, value)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import unittest

from idna.intranges import _encode_range, intranges_contain, intranges_from_list


class IntrangeTests(unittest.TestCase):
    def test_ranging(self):
        self.assertEqual(
            intranges_from_list(list(range(293, 499)) + list(range(4888, 9876))),
            (
                _encode_range(293, 499),
                _encode_range(4888, 9876),
            ),
        )

    def test_ranging_2(self):
        self.assertEqual(intranges_from_list([111]), (_encode_range(111, 112),))

    def test_skips(self):
        self.assertEqual(
            intranges_from_list(
                [
                    0,
                    2,
                    4,
                    6,
                    9,
                    10,
                    11,
                    13,
                    15,
                ]
            ),
            (
                _encode_range(0, 1),
                _encode_range(2, 3),
                _encode_range(4, 5),
                _encode_range(6, 7),
                _encode_range(9, 12),
                _encode_range(13, 14),
                _encode_range(15, 16),
            ),
        )

    def test_empty_range(self):
        self.assertEqual(intranges_from_list([]), ())


class IntrangeContainsTests(unittest.TestCase):
    def _test_containment(self, ints, disjoint_ints):
        ranges = intranges_from_list(ints)
        for int_ in ints:
            assert intranges_contain(int_, ranges)
        for int_ in disjoint_ints:
            assert not intranges_contain(int_, ranges)

    def test_simple(self):
        self._test_containment(range(10, 20), [2, 3, 68, 3893])

    def test_skips(self):
        self._test_containment(
            [
                0,
                2,
                4,
                6,
                9,
                10,
                11,
                13,
                15,
            ],
            [-1, 1, 3, 5, 7, 4898],
        )

    def test_singleton(self):
        self._test_containment([111], [110, 112])

    def test_empty(self):
        self._test_containment([], range(100))



#!/usr/bin/env python

import io
import subprocess
import sys
import unittest
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from unittest import mock

from idna import cli
from idna.package_data import __version__


def run_cli(*argv, stdin=None):
    """Invoke ``cli.main`` with ``argv`` and capture (rc, stdout, stderr).

    Pass ``stdin`` as a string to simulate a piped stdin; omit it to
    simulate an interactive terminal.
    """
    out = io.StringIO()
    err = io.StringIO()
    with redirect_stdout(out), redirect_stderr(err), _patched_stdin(stdin):
        rc = cli.main(list(argv))
    return rc, out.getvalue(), err.getvalue()


class _FakeStdin(io.StringIO):
    """``io.StringIO`` with a configurable ``isatty`` result."""

    def __init__(self, data: str, is_tty: bool) -> None:
        super().__init__(data)
        self._is_tty = is_tty

    def isatty(self) -> bool:
        return self._is_tty


@contextmanager
def _patched_stdin(data):
    """Replace ``sys.stdin`` with a fake stream and isatty() result."""
    stream = _FakeStdin("" if data is None else data, is_tty=data is None)
    with mock.patch.object(sys, "stdin", stream):
        yield


class CLIAutoDetectTests(unittest.TestCase):
    def test_autodetect_encodes_unicode_input(self):
        rc, out, err = run_cli("пример.рф")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "xn--e1afmkfd.xn--p1ai")
        self.assertEqual(err, "")

    def test_autodetect_decodes_alabel(self):
        rc, out, _ = run_cli("xn--e1afmkfd.xn--p1ai")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "пример.рф")

    def test_autodetect_decodes_when_any_label_is_alabel(self):
        # Mixed label: at least one xn-- label triggers decode mode.
        rc, out, _ = run_cli("xn--11b5bs3a9aj6g.example")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "परीक्षा.example")

    def test_autodetect_encodes_plain_ascii_input(self):
        rc, out, _ = run_cli("example.com")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "example.com")

    def test_looks_like_alabel_is_case_insensitive(self):
        self.assertTrue(cli._looks_like_alabel("XN--ZCKZAH"))
        self.assertTrue(cli._looks_like_alabel("foo.Xn--ZcKzAh"))
        self.assertFalse(cli._looks_like_alabel("example.com"))
        self.assertFalse(cli._looks_like_alabel("xnfoo.example"))

    def test_looks_like_alabel_handles_unicode_dots(self):
        # IDEOGRAPHIC FULL STOP should split labels for the purpose of
        # spotting an A-label, matching the behavior of idna.encode/decode.
        self.assertTrue(cli._looks_like_alabel("foo。xn--zckzah"))


class CLIExplicitModeTests(unittest.TestCase):
    def test_encode_flag_forces_encode(self):
        rc, out, _ = run_cli("-e", "παράδειγμα")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "xn--hxajbheg2az3al")

    def test_long_encode_flag_forces_encode(self):
        rc, out, _ = run_cli("--encode", "한국")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "xn--3e0b707e")

    def test_decode_flag_forces_decode(self):
        rc, out, _ = run_cli("-d", "xn--hxajbheg2az3al")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "παράδειγμα")

    def test_long_decode_flag_forces_decode(self):
        rc, out, _ = run_cli("--decode", "xn--3e0b707e")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "한국")

    def test_encode_and_decode_are_mutually_exclusive(self):
        # argparse exits via SystemExit (code 2) for mutually-exclusive errors.
        with self.assertRaises(SystemExit) as ctx, redirect_stderr(io.StringIO()):
            cli.main(["-e", "-d", "example.com"])
        self.assertEqual(ctx.exception.code, 2)


class CLIUTS46Tests(unittest.TestCase):
    def test_uts46_is_applied_by_default(self):
        # Uppercase Greek is not PVALID under IDNA 2008 alone; UTS #46
        # case-folds it so encoding succeeds.
        rc, out, _ = run_cli("ΠΑΡΆΔΕΙΓΜΑ.ΕΛ")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "xn--hxajbheg2az3al.xn--qxam")

    def test_strict_disables_uts46(self):
        rc, out, err = run_cli("--strict", "ΠΑΡΆΔΕΙΓΜΑ.ΕΛ")
        self.assertEqual(rc, 1)
        self.assertEqual(out, "")
        self.assertIn("encode failed", err)
        self.assertIn("U+03A0", err)

    def test_strict_does_not_affect_valid_inputs(self):
        rc, out, _ = run_cli("--strict", "пример.рф")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "xn--e1afmkfd.xn--p1ai")

    def test_uts46_applies_to_explicit_decode(self):
        # Trailing FULLWIDTH FULL STOP (U+FF0E) is mapped to "." by UTS #46
        # so the input still decodes as a single A-label.
        rc, out, _ = run_cli("-d", "xn--zckzah．")
        self.assertEqual(rc, 0)
        self.assertEqual(out.strip(), "テスト.")


class CLIErrorHandlingTests(unittest.TestCase):
    def test_invalid_input_returns_nonzero_and_writes_to_stderr(self):
        rc, out, err = run_cli("foo_bar")
        self.assertEqual(rc, 1)
        self.assertEqual(out, "")
        self.assertTrue(err.startswith("idna: encode failed for 'foo_bar':"))

    def test_invalid_alabel_decode_returns_nonzero(self):
        # Trailing hyphen after the xn-- prefix is rejected by ulabel().
        rc, out, err = run_cli("-d", "xn--")
        self.assertEqual(rc, 1)
        self.assertEqual(out, "")
        self.assertIn("decode failed", err)

    def test_missing_argument_with_tty_stdin_is_an_argparse_error(self):
        # Interactive shell (no piped stdin) and no domain arg → argparse exits 2.
        with self.assertRaises(SystemExit) as ctx, redirect_stderr(io.StringIO()), _patched_stdin(None):
            cli.main([])
        self.assertEqual(ctx.exception.code, 2)


class CLIVersionTests(unittest.TestCase):
    def test_version_flag(self):
        buf = io.StringIO()
        with self.assertRaises(SystemExit) as ctx, redirect_stdout(buf):
            cli.main(["--version"])
        self.assertEqual(ctx.exception.code, 0)
        self.assertIn(__version__, buf.getvalue())


class CLIMultipleDomainsTests(unittest.TestCase):
    def test_multiple_unicode_positional_domains_are_all_encoded(self):
        rc, out, _ = run_cli("пример.рф", "παράδειγμα", "한국")
        self.assertEqual(rc, 0)
        self.assertEqual(
            out.splitlines(),
            ["xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al", "xn--3e0b707e"],
        )

    def test_multiple_alabel_positional_domains_are_all_decoded(self):
        rc, out, _ = run_cli("xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al", "xn--3e0b707e")
        self.assertEqual(rc, 0)
        self.assertEqual(out.splitlines(), ["пример.рф", "παράδειγμα", "한국"])

    def test_mode_flag_applies_to_every_positional_domain(self):
        rc, out, _ = run_cli("-e", "한국", "παράδειγμα")
        self.assertEqual(rc, 0)
        self.assertEqual(
            out.splitlines(),
            ["xn--3e0b707e", "xn--hxajbheg2az3al"],
        )

    def test_failures_are_reported_per_domain_and_processing_continues(self):
        rc, out, err = run_cli("пример.рф", "foo_bar", "παράδειγμα")
        self.assertEqual(rc, 1)
        self.assertEqual(
            out.splitlines(),
            ["xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al"],
        )
        self.assertIn("encode failed for 'foo_bar'", err)


class CLIModeLockTests(unittest.TestCase):
    """The first input picks the mode; the rest of the stream follows."""

    def test_first_unicode_input_locks_subsequent_inputs_to_encode(self):
        # 'foo_bar' is invalid in either direction; the stderr mode word
        # reveals which mode the second input was processed in.
        rc, out, err = run_cli("пример.рф", "foo_bar")
        self.assertEqual(rc, 1)
        self.assertEqual(out.splitlines(), ["xn--e1afmkfd.xn--p1ai"])
        self.assertIn("encode failed for 'foo_bar'", err)

    def test_first_alabel_input_locks_subsequent_inputs_to_decode(self):
        rc, out, err = run_cli("xn--zckzah", "foo_bar")
        self.assertEqual(rc, 1)
        self.assertEqual(out.splitlines(), ["テスト"])
        self.assertIn("decode failed for 'foo_bar'", err)

    def test_explicit_mode_flag_overrides_first_input_heuristic(self):
        # First input looks like an A-label but -e forces encode for everything.
        rc, out, err = run_cli("-e", "xn--zckzah", "foo_bar")
        self.assertEqual(rc, 1)
        self.assertEqual(out.splitlines(), ["xn--zckzah"])
        self.assertIn("encode failed for 'foo_bar'", err)

    def test_stdin_first_line_locks_mode_for_the_stream(self):
        rc, out, err = run_cli(stdin="xn--zckzah\nfoo_bar\n")
        self.assertEqual(rc, 1)
        self.assertEqual(out.splitlines(), ["テスト"])
        self.assertIn("decode failed for 'foo_bar'", err)


class CLIStdinTests(unittest.TestCase):
    def test_reads_one_domain_per_line_from_piped_stdin(self):
        rc, out, _ = run_cli(stdin="пример.рф\nπαράδειγμα\n한국\n")
        self.assertEqual(rc, 0)
        self.assertEqual(
            out.splitlines(),
            ["xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al", "xn--3e0b707e"],
        )

    def test_blank_lines_in_stdin_are_skipped(self):
        rc, out, _ = run_cli(stdin="\nпример.рф\n\n   \nπαράδειγμα\n")
        self.assertEqual(rc, 0)
        self.assertEqual(
            out.splitlines(),
            ["xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al"],
        )

    def test_empty_stdin_exits_cleanly_with_no_output(self):
        rc, out, err = run_cli(stdin="")
        self.assertEqual(rc, 0)
        self.assertEqual(out, "")
        self.assertEqual(err, "")

    def test_positional_domains_take_precedence_over_piped_stdin(self):
        rc, out, _ = run_cli("example.com", stdin="ignored.example\n")
        self.assertEqual(rc, 0)
        self.assertEqual(out.splitlines(), ["example.com"])

    def test_stdin_errors_continue_processing_and_set_exit_code(self):
        rc, out, err = run_cli(stdin="пример.рф\nfoo_bar\nπαράδειγμα\n")
        self.assertEqual(rc, 1)
        self.assertEqual(
            out.splitlines(),
            ["xn--e1afmkfd.xn--p1ai", "xn--hxajbheg2az3al"],
        )
        self.assertIn("encode failed for 'foo_bar'", err)

    def test_decode_flag_applies_to_every_stdin_line(self):
        rc, out, _ = run_cli("-d", stdin="xn--hxajbheg2az3al\nxn--3e0b707e\n")
        self.assertEqual(rc, 0)
        self.assertEqual(out.splitlines(), ["παράδειγμα", "한국"])


class CLIModuleEntryTests(unittest.TestCase):
    def test_python_dash_m_idna_runs_cli(self):
        # End-to-end: spawn ``python -m idna`` and check the round trip.
        result = subprocess.run(
            [sys.executable, "-m", "idna", "xn--zckzah"],
            check=False,
            capture_output=True,
            text=True,
        )
        self.assertEqual(result.returncode, 0)
        self.assertEqual(result.stdout.strip(), "テスト")
        self.assertEqual(result.stderr, "")

    def test_python_dash_m_idna_reads_piped_stdin(self):
        # End-to-end: pipe a list of A-labels and confirm they all decode.
        result = subprocess.run(
            [sys.executable, "-m", "idna"],
            input="xn--e1afmkfd.xn--p1ai\nxn--zckzah\n",
            check=False,
            capture_output=True,
            text=True,
        )
        self.assertEqual(result.returncode, 0)
        self.assertEqual(
            result.stdout.splitlines(),
            ["пример.рф", "テスト"],
        )
        self.assertEqual(result.stderr, "")

    def test_python_dash_m_idna_reports_errors(self):
        result = subprocess.run(
            [sys.executable, "-m", "idna", "foo_bar"],
            check=False,
            capture_output=True,
            text=True,
        )
        self.assertEqual(result.returncode, 1)
        self.assertEqual(result.stdout, "")
        self.assertIn("encode failed", result.stderr)


if __name__ == "__main__":
    unittest.main()
